<?php
// namespace administrator\components\com_jchat\framework\view;
/**
 * @package JCHAT::FRAMEWORK::administrator::components::com_jchat
 * @subpackage framework
 * @subpackage view
 * @author Joomla! Extensions Store
 * @Copyright (C) 2015 - Joomla! Extensions Store
 * @license GNU/GPLv2 http://www.gnu.org/licenses/gpl-2.0.html  
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' );
jimport ( 'joomla.application.component.view' );
jimport ( 'joomla.html.pagination' );
 
/**
 * Base view for all display core
 * 
 * @package JCHAT::FRAMEWORK::administrator::components::com_jchat
 * @subpackage framework
 * @subpackage view
 * @since 1.0
 */
class JChatView extends JViewLegacy {
	/**
	 * Reference to application
	 *
	 * @access public
	 * @var Object
	 */
	public $app;
	
	/**
	 * Reference to option executed
	 *
	 * @access public
	 * @var string
	 */
	public $option;
	/**
	 * Inject language constant into JS Domain maintaining same name mapping
	 * 
	 * @access protected
	 * @param $translations Object&
	 * @param $document Object&
	 * @return void
	 */
	protected function injectJsTranslations($translations, $document) {
		$jsInject = null;
 		// Do translations
		foreach ( $translations as $translation ) {
			$jsTranslation = strtoupper ( $translation );
			$translated = JText::_( $jsTranslation, true );
			$jsInject .= <<<JS
				var $translation = '{$translated}'; 
JS;
		}
		$document->addScriptDeclaration($jsInject);
	}
	
	/**
	 * Manage injecting jQuery framework into document with class inheritance support
	 *
	 * @access protected
	 * @param Object& $doc
	 * @return void
	 */
	protected function loadJQuery($document, $loadJStorage = true) {
		JHtml::_('bootstrap.framework');
		try { JHtml::_('behavior.core'); } catch (Exception $e){} // Compatibility fix ensured for Joomla 3.4+
		
		// jQuery foundation framework
		if($loadJStorage) {
			$document->addScript ( JUri::root ( true ) . '/administrator/components/com_jchat/js/jstorage.min.js' );
		}
		$base = JUri::root();
		$document->addScriptDeclaration("var jchatBaseURI='$base';");
	}
	
	/**
	 * Manage injecting Bootstrap framework into document
	 * 
	 * @access protected
	 * @param Object& $doc
	 * @return void
	 */
	protected function loadBootstrap($document) {
		// Main styles for admin interface
		$document->addStylesheet ( JUri::root ( true ) . '/administrator/components/com_jchat/css/bootstrap-interface.css' );
		
		// Main JS file for admin interface
		$document->addScript ( JUri::root ( true ) . '/administrator/components/com_jchat/js/bootstrap-interface.js' );
	}
	
	/**
	 * Manage injecting valildation plugin into document
	 *
	 * @access protected
	 * @param Object& $doc
	 * @return void
	 */
	protected function loadValidation($document) {
		$document->addStylesheet ( JUri::root ( true ) . '/administrator/components/com_jchat/css/simplevalidation.css' );
		$document->addScript ( JUri::root ( true ) . '/administrator/components/com_jchat/js/jquery.simplevalidation.js' );
	}
	
	/**
	 * Manage injecting valildation plugin into document
	 *
	 * @access protected
	 * @param Object& $doc
	 * @return void
	 */
	protected function loadFrontendValidation($document) {
		$document->addStylesheet ( JUri::root ( true ) . '/components/com_jchat/css/simplevalidation.css' );
		$document->addScript ( JUri::root ( true ) . '/components/com_jchat/js/jquery.simplevalidation.js' );
	}
	
	/**
	 * Manage injecting jQuery UI framework into document
	 *
	 * @access protected
	 * @param Object& $doc
	 * @return void
	 */
	protected function loadJQueryUI($document) {
		$document->addStylesheet ( JUri::root ( true ) . '/administrator/components/com_jchat/css/jqueryui/jquery.ui.all.css' );
		$document->addScript ( JUri::root ( true ) . '/administrator/components/com_jchat/js/jquery.ui.js' );
	}
	
	/**
	 * Manage loading of the Select2 library from CDN
	 *
	 * @access protected
	 * @param Object& $doc
	 * @return void
	 */
	protected function loadSelect2($document) {
		$document->addStylesheet ( 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.8/css/select2.min.css' );
		$document->addScript ( 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.8/js/select2.min.js' );
	}
	
	/**
	 * Class constructor
	 *
	 * @param array $config
	 *        	return Object
	 */
	public function __construct($config = array()) {
		parent::__construct ( $config );
	
		$this->app = JFactory::getApplication ();
		$this->user = JFactory::getUser ();
		$this->doc = JFactory::getDocument();
		$this->option = $this->app->input->get ( 'option' );
	}
}